'****************************************************************************
' MoneyChanger.vbs v20050118
' Copyright (C) Kenneth Gober 2003-2005
' All Rights Reserved
' This software is freely redistributable, with the following restrictions:
'   1. The copyright notice above, and these restrictions, may not be removed.
'   2. This software may not be sold in whole or in part, by itself or in
'      combination with other software, without prior written permission.
'   3. This software may not be included in any collection or archive if such
'      collection is sold for profit, without prior written permission.
'   4. "Prior written permission" may only be granted by the author or his
'      agents.  No waiver of these requirements is granted if you are not
'      able to locate the author or his agents.
'   5. These restrictions do not apply to the authors of SkunkWorks.
'****************************************************************************

Option Explicit

'****************************************************************************
'***                  CUSTOMIZATION SECTION STARTS HERE                   ***
'****************************************************************************

' the exact name and landblock of the buyer who will trade you pyreals for your old currency
' (use /loc to get your current landblock, 'abcd0123' would be entered as '&Habcd0123&'
Const BuyerName = "Archmage Anyao Zai"
Const BuyerLB = &He74e0103&

' the exact name and value of a single item of your old currency
' value refers to the amount you can sell it for, not its face value.
Const BuyerItemName = "Pyreal Pea"
Const BuyerItemValue = 45000
Const BuyerItemStack = 100

' the exact name and landblock of the seller who will trade you new currency for your pyreals
' (use /loc to get your current landblock, 'abcd0123' would be entered as '&Habcd0123&'
Const SellerName = "Archmage Anyao Zai"
Const SellerLB = &He74e0103&

' the exact name and purchase price for a single item of the type you want to accumulate
Const SellerItemName = "Trade Note (100,000)"
Const SellerItemCost = 115000
Const SellerItemStack = 100

' Custom Navigation Section
'  Commands: LogOff, LifestoneRecall, HouseRecall, MansionRecall, UseRouteFile "<file name>"
'            UsePortal "<exact portal name>", CastPortalSpell "<exact spell name>"

Sub GoToBuyer()
  ' do nothing
End Sub

Sub GoToSeller()
  ' do nothing
End Sub

Sub Shutdown()
  ' do nothing
End Sub

'****************************************************************************
'***                   CUSTOMIZATION SECTION ENDS HERE                    ***
'****************************************************************************

' the name of the macro, as indicated by status messages
Const MacroName = "MoneyChanger: "

' filter to locate full stacks of old currency in your inventory
Dim BuyerFullItemFilter
Set BuyerFullItemFilter = skapi.AcfNew()
BuyerFullItemFilter.szName = BuyerItemName
BuyerFullItemFilter.citemStackMin = BuyerItemStack
BuyerFullItemFilter.olc = 1

' filter to locate partial stacks of old currency in your inventory
Dim BuyerPartialItemFilter
Set BuyerPartialItemFilter = skapi.AcfNew()
BuyerPartialItemFilter.szName = BuyerItemName
BuyerPartialItemFilter.citemStackMax = BuyerItemStack - 1
BuyerPartialItemFilter.olc = 1

' filter to locate full stacks of pyreals in any object collection
Dim FullPyFilter
Set FullPyFilter = skapi.AcfNew()
FullPyFilter.szName = "Pyreal"
FullPyFilter.citemStackMin = 10000

' filter to locate partial stacks of pyreals in your inventory
Dim PartialPyFilter
Set PartialPyFilter = skapi.AcfNew()
PartialPyFilter.szName = "Pyreal"
PartialPyFilter.citemStackMax = 9999
PartialPyFilter.olc = 1

' filter to locate new currency in any object collection
Dim SellerItemFilter
Set SellerItemFilter = skapi.AcfNew()
SellerItemFilter.szName = SellerItemName

' filter to locate partial stacks of new currency in your inventory
Dim SellerPartialItemFilter
Set SellerPartialItemFilter = skapi.AcfNew()
SellerPartialItemFilter.szName = SellerItemName
SellerPartialItemFilter.citemStackMax = SellerItemStack - 1
SellerPartialItemFilter.olc = 1

' main pack = yourself
Dim MainPack
Set MainPack = skapi.acoChar

' event/message handler
Dim Handler
Call InitHandler()

' interval timer for action timeouts
Dim T
Set T = skapi.TimerNew()

' interval timer for Buy category
Dim BuyTimer
Set BuyTimer = skapi.TimerNew()
BuyTimer.tag = 0

' total runtime timer
Dim Clock
Set Clock = skapi.TimerNew()

' running trade volume measured in pyreals, old currency and new currency
Dim PyrealVolume, SellerItemVolume, BuyerItemVolume

' pixel tests for chat window size
Dim ChatExpanded
Set ChatExpanded = skapi.PxtNew(0, -92, 2064, 6218)

' pixel tests for green glow under name of selected item (lower right corner of AC UI)
Dim CurrentSelectionMulti, CurrentSelectionSingleOK, CurrentSelectionMultiOK
Set CurrentSelectionMulti = skapi.PxtNew(-231, -62, 0, 3223601)
Set CurrentSelectionSingleOK = skapi.PxtNew(-169, -61, 1588248, 0)
Set CurrentSelectionMultiOK = skapi.PxtNew(-213, -36, 3239225, 528408)

' SkunkWorks will call this procedure after loading the script
Sub Main()
  Dim s
  s = 0				' start in state 0 (decide where to go next)
  Do
    Select Case (s)
      Case 0: s = Part0()	' decide where to go next
      Case 1: s = Part1()	' sell old currency
      Case 2: s = Part2()	' stack pyreals
      Case 3: s = Part3()	' buy new currency
    End Select
  Loop Until (s = -1)		' exit script when state becomes -1
End Sub

' state 0 - decide where to go next
Function Part0()
  If ((skapi.cpyCash \ SellerItemCost) > 0) Then
    Call NavigateSeller()				' if enough cash to buy new currency, go to seller
    Part0 = 3						' and set next state to 'buy new currency'
  ElseIf (skapi.CitemOfKindInInv(BuyerItemName) > 0) Then
    Call NavigateBuyer()				' if any old currency on hand, go to buyer
    Part0 = 1						' and set next state to 'sell old currency'
  Else
    Call NavigateShutdown()				' if no remaining old currency, go to shutdown point
    Part0 = -1						' and set next state to 'terminate'
  End If
End Function

' state 1 - sell old currency
Function Part1()
  Dim merchant, coll, obj, py, k, n, ct
  Do							' approach merchant
    Set merchant = skapi.AcoFromSz(BuyerName)
    Handler.State = &H10000&
    T.cmsec = 0
    If Not (merchant Is Nothing) Then
      Call merchant.Use()
      Do
        Call skapi.WaitEvent()
      Loop Until (Handler.State = &H10001&) Or (T.cmsec > 5000)
    End If
  Loop Until (Handler.State = &H10001&)
  Call skapi.Keys("{F11}")
  Call skapi.Keys("{F12}")
  Call MouseClickExpandChat()
  py = 0
  ct = 0
  k = ((SlotsAvailable(MainPack) - 2) * 10000) \ BuyerItemValue
  Set coll = BuyerPartialItemFilter.CoacoGet()		' locate partial stacks of old currency in main pack
  For Each obj In coll
    n = obj.citemStack
    If (n > k) Then
      Call skapi.SelectAco(obj)
      Call skapi.SetStackCount(k)
      Handler.State = &H20000&
      T.cmsec = 0
      Call obj.MoveToPack(0, 0, False)
      Do
        Call skapi.WaitEvent()
      Loop Until (Handler.State = &H20007&)
      BuyerPartialItemFilter.citemStackMin = k
      BuyerPartialItemFilter.citemStackMax = k
      Set obj = BuyerPartialItemFilter.CoacoGet().Item(0)
      BuyerPartialItemFilter.citemStackMax = BuyerItemStack - 1
      n = k
    End If
    Call obj.MoveToStack(merchant)
    k = k - n
    py = py + n * BuyerItemValue
    ct = ct + n
    If (k <= 0) Then Exit For
  Next
  If (k > 0) Then
    Set coll = BuyerFullItemFilter.CoacoGet()		' locate full stacks of old currency in main pack
    For Each obj In coll
      n = obj.citemStack
      If (n > k) Then
        Call skapi.SelectAco(obj)
        Call skapi.SetStackCount(k)
        Handler.State = &H20000&
        T.cmsec = 0
        Call obj.MoveToPack(0, 0, False)
        Do
          Call skapi.WaitEvent()
        Loop Until (Handler.State = &H20007&)
        BuyerPartialItemFilter.citemStackMin = k
        BuyerPartialItemFilter.citemStackMax = k
        Set obj = BuyerPartialItemFilter.CoacoGet().Item(0)
        BuyerPartialItemFilter.citemStackMax = BuyerItemStack - 1
        n = k
      End If
      Call obj.MoveToStack(merchant)
      k = k - n
      py = py + n * BuyerItemValue
      ct = ct + n
      If (k <= 0) Then Exit For
    Next
  End If
  PyrealVolume = PyrealVolume + py
  BuyerItemVolume = BuyerItemVolume + ct
  py = py + skapi.cpyCash
  Handler.State = &H50000&
  T.cmsec = 0
  Call MouseClickSellAll()
  Do
    Call skapi.WaitEvent()
  Loop Until (Handler.State = &H50003&) And (skapi.cpyCash >= py) Or (T.cmsec > 5000)
  Set obj = Nothing
  Set coll = Nothing
  Set merchant = Nothing
  Part1 = 2						' next state is to stack pyreals
End Function

' state 2 - stack pyreals
Function Part2()
  Dim coll, obj, dest, n, k
  Set coll = FullPyFilter.CoacoGet(MainPack.coacoContents)
  Set dest = FindEmptyPack()
  For Each obj In coll					' move full stacks to side packs 
    If (dest Is MainPack) Then Exit For
    Handler.State = &H60000&
    T.cmsec = 0
    Call obj.MoveToContainer(dest, 0, False)
    Do
      Call skapi.WaitEvent()
    Loop Until (Handler.State = &H60001&) Or (T.cmsec > 2000)
    If (SlotsAvailable(dest) = 0) Then Set dest = FindEmptyPack()
  Next
  Set coll = PartialPyFilter.CoacoGet()
  Do While (coll.Count > 1)				' merge partial stacks
    Set obj = coll.Item(0)
    Set dest = coll.Item(1)
    n = dest.citemMaxStack - dest.citemStack
    k = obj.citemStack
    Handler.State = &H70000&
    T.cmsec = 0
    Call obj.MoveToStack(dest)
    Do
      Call skapi.WaitEvent()
    Loop Until (Handler.State = &H70001&) Or (T.cmsec > 2000)
    If (k <= n) Then Call coll.RemoveAco(obj)
    If (k >= n) Then Call coll.RemoveAco(dest)
  Loop
  Part2 = 0						' next state is to decide where to go
End Function

' state 3 - buy new currency
Function Part3()
  Dim c, n, k, coll, obj, dest
  Do							' approach merchant
    Set obj = skapi.AcoFromSz(SellerName)
    Handler.State = &H10000&
    T.cmsec = 0
    If Not (obj Is Nothing) Then
      Call obj.Use()
      Do
        Call skapi.WaitEvent()
      Loop Until (Handler.State = &H10001&) Or (T.cmsec > 5000)
    End If
  Loop Until (Handler.State = &H10001&)
  c = skapi.cpyCash \ SellerItemCost
  Call Buy(SellerItemName, SellerItemCost, c)
  SellerItemVolume = SellerItemVolume + c
  If (SellerItemStack > 1) Then
    Set coll = SellerPartialItemFilter.CoacoGet()
    Do While (coll.Count > 1)				' stack items
      Set obj = coll.Item(0)
      Set dest = coll.Item(1)
      n = dest.citemMaxStack - dest.citemStack
      k = obj.citemStack
      Handler.State = &H70000&
      T.cmsec = 0
      Call obj.MoveToStack(dest)
      Do
        Call skapi.WaitEvent()
      Loop Until (Handler.State = &H70001&) Or (T.cmsec > 2000)
      If (k <= n) Then Call coll.RemoveAco(obj)
      If (k >= n) Then Call coll.RemoveAco(dest)
    Loop
  End If
  Set coll = SellerItemFilter.CoacoGet(MainPack.coacoContents)
  Set dest = FindEmptyPack()
  For Each obj In coll					' move items to side packs
    If (dest Is MainPack) Then Exit For
    Handler.State = &H60000&
    T.cmsec = 0
    Call obj.MoveToContainer(dest)
    Do
      Call skapi.WaitEvent()
    Loop Until (Handler.State = &H60001&) Or (T.cmsec > 3000)
    If (SlotsAvailable(dest) = 0) Then Set dest = FindEmptyPack()
  Next
  Set dest = Nothing
  Set obj = Nothing
  Set coll = Nothing
  Part3 = 0						' next state is to decide where to go
End Function

Sub LogOff()
  Call skapi.MouseMove(skapi.dxpWindow - 13, 13)	' click X at upper right corner
  Call skapi.MouseButton(0)
  Call skapi.Sleep(250)
  Call skapi.Keys("y")					' and answer (Y)es
End Sub

Sub NavigateBuyer()
  If (MainPack.maploc.landblock <> BuyerLB) Then
    Call skapi.OutputLine(MacroName & "Going to buyer.")
    Call GoToBuyer()
  End If
End Sub

Sub NavigateSeller()
  If (MainPack.maploc.landblock <> SellerLB) Then
    Call skapi.OutputLine(MacroName & "Going to seller.")
    Call GoToSeller()
  End If
End Sub

Sub NavigateShutdown()
  Call skapi.OutputLine(MacroName & "Going to shutdown.")
  Call Shutdown()
End Sub

Sub UseRouteFile(name)
  Call RouteFromFile(name).Go()
End Sub

Sub UsePortal(name)
  Dim lb, obj, r, ct
  lb = MainPack.maploc.landblock
  ct = 0
  Do
    Set obj = skapi.AcoFromSz(name)
    If Not (obj Is Nothing) Then
      T.cmsec = 0
      Do
        Call obj.Use()
        r = skapi.WaitEvent(2000, 2, 17)
      Loop Until (r = 17) Or (T.cmsec > 20000)
      If (r = 17) Then Call skapi.WaitEvent(60000, 2, 18)
      Set obj = Nothing
    End If
    Call skapi.WaitEvent(1000)
    If (MainPack.maploc.landblock = lb) Then
      If (ct < 3) Then
        Call skapi.OutputLine(MacroName & "It doesn't look like I went anywhere.  I'll try again.")
        ct = ct + 1
      Else
        Call skapi.OutputLine(MacroName & "It doesn't look like I went anywhere, but I'll proceed anyway.")
        Exit Do
      End If
    End If
  Loop Until (MainPack.maploc.landblock <> lb)
End Sub

Sub CastPortalSpell(name)
  Dim s, lb, r, ct
  Set s = skapi.SpellInfoFromSpellid(skapi.SpellidFromSz(name))
  lb = MainPack.maploc.landblock
  ct = 0
  Do
    Do
      If (skapi.manaCur < s.mana) Then
        Call skapi.OutputLine(MacroName & "Not enough mana to cast spell, resting.")
        Call SetCombatMode(false)
        Call skapi.Keys("b")
        Do Until (skapi.manaCur >= s.mana)
          Call skapi.WaitEvent(5000)
        Loop
      End If
      Call SetCombatMode(true)
      r = skapi.arc
      Handler.State = &H90000&
      Call skapi.CastSpell(s.spellid)
      T.cmsec = 0
      Do
        Call skapi.WaitEvent()
        r = skapi.arc
      Loop Until (r <> 0) Or (T.cmsec > 6000) Or (Handler.State = &H90001&)
    Loop Until (Handler.State = &H90001&)
    Call skapi.WaitEvent(60000, 2, 18)
    Call skapi.WaitEvent(1000)
    If (MainPack.maploc.landblock = lb) Then
      If (ct < 3) Then
        Call skapi.OutputLine(MacroName & "It doesn't look like I went anywhere.  I'll try again.")
        ct = ct + 1
      Else
        Call skapi.OutputLine(MacroName & "It doesn't look like I went anywhere, but I'll proceed anyway.")
        Exit Do
      End If
    End If
  Loop Until (MainPack.maploc.landblock <> lb)
  Call SetCombatMode(false)
  Set s = Nothing
End Sub

Sub LifestoneRecall()
  Dim lb, ct
  lb = MainPack.maploc.landblock
  ct = 0
  Do
    Call skapi.Keys("{Return}")
    Call skapi.PasteSz("/lifestone")
    Call skapi.Keys("{Return}")
    Call skapi.WaitEvent(90000, 2, 18)
    Call skapi.WaitEvent(1000)
    If (MainPack.maploc.landblock = lb) Then
      If (ct < 3) Then
        Call skapi.OutputLine(MacroName & "It doesn't look like I went anywhere.  I'll try again.")
        ct = ct + 1
      Else
        Call skapi.OutputLine(MacroName & "It doesn't look like I went anywhere, but I'll proceed anyway.")
        Exit Do
      End If
    End If
  Loop Until (MainPack.maploc.landblock <> lb)
End Sub

Sub HouseRecall()
  Dim lb, ct
  lb = MainPack.maploc.landblock
  ct = 0
  Do
    Call skapi.Keys("{Return}")
    Call skapi.PasteSz("/house recall")
    Call skapi.Keys("{Return}")
    Call skapi.WaitEvent(90000, 2, 18)
    Call skapi.WaitEvent(1000)
    If (MainPack.maploc.landblock = lb) Then
      If (ct < 3) Then
        Call skapi.OutputLine(MacroName & "It doesn't look like I went anywhere.  I'll try again.")
        ct = ct + 1
      Else
        Call skapi.OutputLine(MacroName & "It doesn't look like I went anywhere, but I'll proceed anyway.")
        Exit Do
      End If
    End If
  Loop Until (MainPack.maploc.landblock <> lb)
End Sub

Sub MansionRecall()
  Dim lb, ct
  lb = MainPack.maploc.landblock
  ct = 0
  Do
    Call skapi.Keys("{Return}")
    Call skapi.PasteSz("/house mansion_recall")
    Call skapi.Keys("{Return}")
    Call skapi.WaitEvent(90000, 2, 18)
    Call skapi.WaitEvent(1000)
    If (MainPack.maploc.landblock = lb) Then
      If (ct < 3) Then
        Call skapi.OutputLine(MacroName & "It doesn't look like I went anywhere.  I'll try again.")
        ct = ct + 1
      Else
        Call skapi.OutputLine(MacroName & "It doesn't look like I went anywhere, but I'll proceed anyway.")
        Exit Do
      End If
    End If
  Loop Until (MainPack.maploc.landblock <> lb)
End Sub

Sub Buy(name, cost, count)
  Dim markup, mcoll, obj, filt, coll, c, ct
  Dim tArmor, tBooks, tClothing, tContainers, tFood, tGems, tJewelry, tKeys, tMisc
  Dim tServices, tComps, tNotes, tWeapons, tStones, tMagic, tAlch, tCook, tFletch
  markup = Handler.LastMerchantMarkup
  If (LCase(Left(name, 10)) = "trade note") Then markup = 1.15
  Set mcoll = Handler.LastMerchantCoaco
  For Each obj In mcoll					' enumerate all categories in stock
    Select Case (obj.szMcm)
      Case "Armor": tArmor = 1
      Case "Books, Paper": tBooks = 1
      Case "Clothing": tClothing = 1
      Case "Containers": tContainers = 1
      Case "Food": tFood = 1
      Case "Gems": tGems = 1
      Case "Jewelry": tJewelry = 1
      Case "Keys, Tools": tKeys = 1
      Case "Miscellaneous": tMisc = 1
      Case "Services": tServices = 1
      Case "Spell Components": tComps = 1
      Case "Trade Notes": tNotes = 1
      Case "Weapons": tWeapons = 1
      Case "Mana Stones": tStones = 1
      Case "Magic Items": tMagic = 1
      Case "Alchemical Items": tAlch = 1
      Case "Cooking Items": tCook = 1
      Case "Fletching Items": tFletch = 1
    End Select
  Next
  Set filt = skapi.AcfNew()				' locate item to be purchased
  filt.szName = name
  filt.cpyMin = Int(cost / markup)
  filt.cpyMax = Ceil(cost / markup)
  Set coll = filt.CoacoGet(mcoll)
  If (coll.Count = 0) Then
    Call skapi.OutputLine(MacroName & "This merchant doesn't sell " & name & "s for " & FormatNumber(cost, 0) & " pyreals.")
  Else
    Set obj = coll.Item(0)				' locate which line in the merchant category list to choose
    Select Case (obj.szMcm)
      Case "Armor": c = tArmor
      Case "Books, Paper": c = tArmor + tBooks
      Case "Clothing": c = tArmor + tBooks + tClothing
      Case "Containers": c = tArmor + tBooks + tClothing + tContainers
      Case "Food": c = tArmor + tBooks + tClothing + tContainers + tFood
      Case "Gems": c = tArmor + tBooks + tClothing + tContainers + tFood + tGems
      Case "Jewelry": c = tArmor + tBooks + tClothing + tContainers + tFood + tGems + tJewelry
      Case "Keys, Tools": c = tArmor + tBooks + tClothing + tContainers + tFood + tGems + tJewelry + tKeys
      Case "Miscellaneous": c = tArmor + tBooks + tClothing + tContainers + tFood + tGems + tJewelry + tKeys + tMisc
      Case "Services": c = tArmor + tBooks + tClothing + tContainers + tFood + tGems + tJewelry + tKeys + tMisc + tServices
      Case "Spell Components": c = tArmor + tBooks + tClothing + tContainers + tFood + tGems + tJewelry + tKeys + tMisc + tServices + tComps
      Case "Trade Notes": c = tArmor + tBooks + tClothing + tContainers + tFood + tGems + tJewelry + tKeys + tMisc + tServices + tComps + tNotes
      Case "Weapons": c = tArmor + tBooks + tClothing + tContainers + tFood + tGems + tJewelry + tKeys + tMisc + tServices + tComps + tNotes + tWeapons
      Case "Mana Stones": c = tArmor + tBooks + tClothing + tContainers + tFood + tGems + tJewelry + tKeys + tMisc + tServices + tComps + tNotes + tWeapons + tStones
      Case "Magic Items": c = tArmor + tBooks + tClothing + tContainers + tFood + tGems + tJewelry + tKeys + tMisc + tServices + tComps + tNotes + tWeapons + tStones + tMagic
      Case "Alchemical Items": c = tArmor + tBooks + tClothing + tContainers + tFood + tGems + tJewelry + tKeys + tMisc + tServices + tComps + tNotes + tWeapons + tStones + tMagic + tAlch
      Case "Cooking Items": c = tArmor + tBooks + tClothing + tContainers + tFood + tGems + tJewelry + tKeys + tMisc + tServices + tComps + tNotes + tWeapons + tStones + tMagic + tAlch + tCook
      Case "Fletching Items": c = tArmor + tBooks + tClothing + tContainers + tFood + tGems + tJewelry + tKeys + tMisc + tServices + tComps + tNotes + tWeapons + tStones + tMagic + tAlch + tCook + tFletch
    End Select
    Call skapi.Keys("{F11}")				' set up screen so that buttons are at expected coordinates
    Call skapi.Keys("{F12}")
    Call MouseClickExpandChat()
    Call MouseClickMerchantTab(1)
    If (BuyTimer.tag <> c) Or (BuyTimer.cmsec > 5000) Then
      Call MouseClickMerchantCategoryButton()	' choose proper category
      Call MouseClickMerchantCategory(c)
    End If
    BuyTimer.tag = c
    ct = count
    Do While (ct > 0)
      c = Min(ct, obj.citemMaxStack)
      Call skapi.SelectAco(obj)				' select item
      Call skapi.Sleep(200)
      If (c > 1) Then
        Call skapi.Keys("t")					' specify how many
        Call skapi.Keys(CStr(c))
      End If
      ct = ct - c
      Handler.State = &H80000&
      Handler.BuyName = name
      T.cmsec = 0
      Call MouseClickBuyItem()				' buy
      Do
        Call skapi.WaitEvent()
      Loop Until (Handler.State = &H80003&) Or (T.cmsec > 5000)
    Loop
    BuyTimer.cmsec = 0
  End If
  Set coll = Nothing
  Set filt = Nothing
  Set obj = Nothing
  Set mcoll = Nothing
End Sub

Class MessageHandler
  Private mState
  Private mLastMerchantCoaco
  Private mLastMerchantMarkup
  Private mBuyName

  Private Sub Class_Initialize()
  End Sub

  Private Sub Class_Terminate()
    Set mLastMerchantCoaco = Nothing
  End Sub

  Public Property Get State()
    State = mState
  End Property

  Public Property Let State(RHS)
    mState = RHS
  End Property

  Public Property Get LastMerchantCoaco()
    Set LastMerchantCoaco = mLastMerchantCoaco
  End Property

  Public Property Get LastMerchantMarkup()
    LastMerchantMarkup = mLastMerchantMarkup
  End Property

  Public Property Let BuyName(RHS)
    mBuyName = RHS
  End Property

  Public Sub OnActionComplete()
    Select Case (mState And &H0FFF0000&)
      Case &H40000&
        mState = mState Or 1
    End Select
  End Sub

  Public Sub OnApproachVendor(acoMerchant, mcmBuy, cpyMaxBuy, fractBuy, fractSell, coaco)
    mLastMerchantMarkup = fractSell
    Set mLastMerchantCoaco = coaco
    Select Case (mState And &H0FFF0000&)
      Case &H10000&
        mState = mState Or 1
    End Select
  End Sub

  Public Sub OnObjectCreate(aco)
    Select Case (mState And &H0FFF0000&)
      Case &H20000&
        If (aco.szName = BuyerItemName) Then mState = mState Or 2
      Case &H30000&
        If (aco.szName = Item2Name) Then mState = mState Or 2
      Case &H50000&
        If (aco.szName = "Pyreal") Then mState = mState Or 2
      Case &H80000&
        If (aco.szName = mBuyName) Then mState = mState Or 2
    End Select
  End Sub

  Public Sub OnAddToInventory(aco)
    Select Case (mState And &H0FFF0000&)
      Case &H20000&
        mState = mState Or 1
      Case &H30000&
        mState = mState Or 1
      Case &H50000&
        mState = mState Or 1
      Case &H60000&
        mState = mState Or 1
      Case &H80000&
        mState = mState Or 1
    End Select
  End Sub

  Public Sub OnAdjustStack(aco, citemPrev)
    Select Case (mState And &H0FFF0000&)
      Case &H20000&
        mState = mState Or 4
      Case &H70000&
        mState = mState Or 1
    End Select
  End Sub

  Public Sub OnStartPortalSelf()
    Select Case (mState And &H0FFF0000&)
      Case &H90000&
        mState = mState Or 1
    End Select
  End Sub

  Public Sub OnCommand(szCmd)
    szCmd = LCase(Trim(szCmd))
    If (szCmd = "status") Or (szCmd = "report") Then
      Call skapi.OutputLine(MacroName & "Total transaction volume: " & FormatNumber(PyrealVolume, 0) & " pyreals in " & Duration(Clock.cmsec))
      Call skapi.OutputLine(MacroName & "Bought: " & FormatNumber(SellerItemVolume, 0) & " " & SellerItemName & "s total.")
      Call skapi.OutputLine(MacroName & "Sold: " & FormatNumber(BuyerItemVolume, 0) & " " & BuyerItemName & "s total.")
    End If
  End Sub
End Class

Sub InitHandler()
  Set Handler = New MessageHandler
  Call skapi.AddHandler(9, Handler)
  Call skapi.AddHandler(17, Handler)
  Call skapi.AddHandler(23, Handler)
  Call skapi.AddHandler(49, Handler)
  Call skapi.AddHandler(52, Handler)
  Call skapi.AddHandler(63, Handler)
  Call skapi.AddHandler(66, Handler)
End Sub

Function Min(a, b)
  If (a <= b) Then
    Min = a
  Else
    Min = b
  End If
End Function

Function Max(a, b)
  If (a >= b) Then
    Max = a
  Else
    Max = b
  End If
End Function

Function Ceil(x)
  If (x = Int(x)) Then
    Ceil = x
  Else
    Ceil = Int(x + 1)
  End If
End Function

Function Duration(msec)
  Dim buf, t
  t = msec \ 1000
  buf = Right("0" & (t Mod 60), 2)
  t = t \ 60
  buf = Right("0" & (t Mod 60), 2) & ":" & buf
  t = t \ 60
  Duration = t & ":" & buf
End Function

Function FindEmptyPack()
  Dim c, pack
  c = 7
  Do
    Set pack = skapi.AcoFromIpackIitem(c, -1)
    c = c - 1
  Loop Until (SlotsAvailable(pack) > 0) Or (c < 0)
  Set FindEmptyPack = pack
  Set pack = Nothing
End Function

Function SlotsAvailable(pack)
  If (pack Is Nothing) Then
    SlotsAvailable = 0
  ElseIf (pack.citemMax = 0) Then
    SlotsAvailable = 0
  Else
    SlotsAvailable = pack.citemMax - pack.citemContents
  End If
End Function

Function CurrentSelectionOK()
  If (CurrentSelectionMulti.FCheck()) Then
    CurrentSelectionOK = CurrentSelectionMultiOK.FCheck()
  Else
    CurrentSelectionOK = CurrentSelectionSingleOK.FCheck()
  End If
End Function

Sub SetCombatMode(mode)
  Do Until (skapi.fCombatMode = mode)
    Call skapi.Keys("`")
    T.cmsec = 0
    Do
      Call skapi.WaitEvent()
    Loop Until (skapi.fCombatMode = mode) Or (T.cmsec > 2000)
    If Not (skapi.fCombatMode = mode) Then Call skapi.Sleep(2000)
  Loop
End Sub

Sub MouseClickExpandChat()
  If Not (ChatExpanded.FCheck()) Then
    Call skapi.MouseMove(skapi.dxpWindow - 318, skapi.dypWindow - 94)
    Call skapi.MouseButton(0)
    Call skapi.Sleep(100)
  End If
End Sub

Sub MouseClickMerchantTab(n)
  If (n < 1) Or (n > 3) Then Exit Sub
  Call skapi.MouseMove(230 - (92 * (3 - n)), (0.75 * height) - 161)
  Call skapi.MouseButton(0)
  Call skapi.Sleep(100)
End Sub

Sub MouseClickBuyItem()
  Call skapi.MouseMove(skapi.dxpWindow - 340, (0.75 * skapi.dypWindow) - 134)
  Call skapi.MouseButton(0)
  Call skapi.Sleep(100)
End Sub

Sub MouseClickSellAll()
  Call skapi.MouseMove(skapi.dxpWindow - 340, (0.75 * skapi.dypWindow) - 102)
  Call skapi.MouseButton(0)
  Call skapi.Sleep(100)
End Sub

Sub MouseClickMerchantCategoryButton()
  Do Until (CurrentSelectionOK())
    Call skapi.Sleep(50)
  Loop
  Call skapi.MouseMove(113, (0.75 * skapi.dypWindow) - 136)
  Call skapi.MouseButton(0)
  Call skapi.Sleep(100)
End Sub

Sub MouseClickMerchantCategory(n)
  If (n < 1) Or (n > 12) Then Exit Sub
  Call skapi.MouseMove(55, (0.75 * skapi.dypWindow) - 136 + (18 * n))
  Call skapi.MouseButton(0)
  Call skapi.Sleep(100)
End Sub
